unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ScktComp, ShellApi, Menus, RasApi, IniFiles;

const
  WM_TASKBAREVENT = WM_USER + 20;
  MAX_RASCONNECTIONS = 10;

type
  TMainForm = class(TForm)
    GroupBox1: TGroupBox;
    EditPort: TEdit;
    Label1: TLabel;
    GroupBox2: TGroupBox;
    ServerStatusLabel: TLabel;
    BtnServerStart: TButton;
    BtnServerStop: TButton;
    chkAutostart: TCheckBox;
    HTTPServer: TServerSocket;
    GroupBox3: TGroupBox;
    Label2: TLabel;
    BoxConnections: TComboBox;
    Label3: TLabel;
    EditUsername: TEdit;
    EditPassword: TEdit;
    Label4: TLabel;
    BtnOK: TButton;
    BtnCancel: TButton;
    PopupMenu1: TPopupMenu;
    Show1: TMenuItem;
    N1: TMenuItem;
    Beenden1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure Show1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BoxConnectionsChange(Sender: TObject);
    procedure EditPasswordChange(Sender: TObject);
    procedure EditUsernameChange(Sender: TObject);
    procedure Beenden1Click(Sender: TObject);
    procedure HTTPServerClientConnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure BtnServerStartClick(Sender: TObject);
    procedure BtnServerStopClick(Sender: TObject);
    procedure HTTPServerClientRead(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure BtnCancelClick(Sender: TObject);
    procedure BtnOKClick(Sender: TObject);
  private
    { Private declarations }
    NotifyIconData: TNotifyIconData; (* speichert das TrayIcon *)
    RasDialParams: TRasDialParams; (* speichert die Whl-Parameter,
                        wie PhoneNumber, Username, Password, ... *)
    hRAS: THRasConn;  (* speichert das Handle einer vom Programm
                         aufgebauten Verbindung oder das Handle einer
                         aktiven Verbindung der letzten Status-Abfrage;
                         keine Verbindung hRAS = 0 *)

    indata: String; (* speichert die empfangene Zeile -> OnClientRead *)
    error: Integer; (* speichert den letzten aufgetrenenen Fehler
                        oder SUCCESS=0 *)

    function Dial : Boolean;
    function Status(var entryname: String) : Boolean;
    function Hangup : Boolean;
    function getRASError(code: Integer) : String;

    procedure SendStatus(Socket: TCustomWinSocket);
    procedure SendOnline(Socket: TCustomWinSocket);
    procedure SendOffline(Socket: TCustomWinSocket);
    procedure SendAbout(Socket: TCustomWinSocket);
    procedure SendFileNotFound(Socket: TCustomWinSocket);

    procedure SaveSettings;
    procedure LoadSettings;

    procedure TaskbarEvent(var Msg: TMessage);
      Message WM_TASKBAREVENT;
    procedure WMSysCommand(var Message: TWMSysCommand);
      Message WM_SYSCOMMAND;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.TaskbarEvent(var Msg: TMessage);
var Point: TPoint;
begin
  { Die WM_TaskbarEvent-Message "Msg" gibt in Msg.LParam
    das genaue Ereignis an. Msg.LParam kann folgende Werte fr
    Mausereignisse annehmen:

    WM_MouseMove
    WM_LButtonDown
    WM_LButtonUp
    WM_LButtonDblClk
    WM_RButtonDown
    WM_RButtonUp
    WM_RButtonDblClk }

    (* Behandelt die Ereignisse des TrayIcons *)

    if (Msg.LParam = WM_LButtonDown) then begin
      if Visible then Hide
      else begin
        Show;
        Application.BringToFront;
        SetFocus;
      end;
    end
    else if (Msg.LParam = WM_RButtonDown) then begin
      GetCursorPos(Point);
      PopupMenu1.Popup(Point.X, Point.Y);
    end;
end;

procedure TMainForm.WMSysCommand(var Message: TWMSysCommand);
begin
  (* Wenn MainForm minimiert wird, wird es versteckt...
     es erscheint nicht in der Taskbar *)
  if Message.CmdType AND $FFF0 = SC_MINIMIZE then
    Hide
  else
    inherited;
end;


procedure TMainForm.FormCreate(Sender: TObject);
var entrynamesArray: Array[1..MAX_RASCONNECTIONS] of TRasEntryName;
    i,ret,lpcb: Integer;
    lpcEntries: Integer;
begin
  (* TrayIcon erstellen *)
  NotifyIconData.cbSize := SizeOf(NotifyIconData);
  NotifyIconData.Wnd := Handle;
  NotifyIConData.uID := 1;
  NotifyIconData.uFlags := NIF_MESSAGE + NIF_ICON + NIF_TIP;
  NotifyIconData.uCallbackMessage := WM_TASKBAREVENT;
  NotifyIconData.hIcon := Application.Icon.Handle;
  NotifyIconData.szTip := ' RAS Manager ';
  (* TrayIcon installieren *)
  Shell_NotifyIcon(NIM_ADD, @NotifyIconData);

  (* DF-Verbindungen suchen *)
  for i := 1 to MAX_RASCONNECTIONS do begin
    FillChar(entrynamesArray[i], SizeOf(TRasEntryName), 0);
    entrynamesArray[i].dwSize := SizeOf(TRasEntryName);
  end;
  lpcb := SizeOf(TRasEntryName) * MAX_RASCONNECTIONS;
  lpcEntries := 0;
  ret := RasEnumEntries(nil, nil, @entrynamesArray, lpcb, lpcEntries);

  if (ret = SUCCESS) then begin

    (* DF-Verbindungen in die ComboBox einfgen *)
    for i := 1 to lpcEntries do begin
      BoxConnections.Items.Add(entrynamesArray[i].szEntryName);
    end;
    BoxConnections.ItemIndex := 0;
    BoxConnectionsChange(nil);
  end;

  (* Verbindungshandle auf 0 stellen -> keine Verbindung *)
  hRAS := 0;
  (* Fehler besteht bis jetzt noch nicht ;-) *)
  error := SUCCESS;


  (* Einstellungen laden *)
  LoadSettings;

  (* Server sofort starten? *)
  if chkAutostart.Checked = True then begin
    BtnServerStartClick(Sender);
    (* minimieren... *)
    Application.ShowMainForm := False;
    Hide;  
  end;

  (* Zufallsgenerator initialisieren *)
  Randomize;
end;

procedure TMainForm.Show1Click(Sender: TObject);
begin
  Show;
  Application.BringToFront;
  SetFocus;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  (* TrayIcon zerstren *)
  Shell_NotifyIcon(NIM_DELETE, @NotifyIconData);
end;

procedure TMainForm.BoxConnectionsChange(Sender: TObject);
var ret: Integer;
    lpfPassword: LongBool;
begin
  (* DF-Whlparameter fr ausgewhlte Verbindung holen *)
  FillChar(RasDialParams, SizeOf(TRasDialParams), 0);
  RasDialParams.dwSize := SizeOf(TRasDialParams);
  StrPCopy(RasDialParams.szEntryName, BoxConnections.Items[BoxConnections.ItemIndex]);

  ret := RasGetEntryDialParams(nil, RasDialParams, lpfPassword);

  if (ret = SUCCESS) then begin
    (* Benutzername anzeigen *)
    EditUsername.Text := RasDialParams.szUserName;
    (* Passwort muss eingegeben werden;
       nur Win9x liefert Passwort im Klartext in RasDialParams zurck,
       Win2000 liefert nur **** *)
    (* Im Prinzip gibt lpfPassword an, ob ein Passwort gespeichert
       ist. Das ist dann auch in RasDialParams.szPassword *)

    (* leeres Passwort... *)
    EditPassword.Text := '';
    EditPasswordChange(Sender);
  end;
end;

procedure TMainForm.EditPasswordChange(Sender: TObject);
begin
  (* Passwort-nderung in den WhlParametern speichern *)
  StrPCopy(RasDialParams.szPassword, EditPassword.Text);
end;

function TMainForm.Dial : Boolean;
begin
  error := RasDial(nil, nil, RasDialParams, 0, nil, hRAS);
  if (error <> SUCCESS) then begin
    //ShowMessage('Fehler beim Verbindungsaufbau: ' + IntToStr(error));
    Result := False;
    Exit;
  end;
  Result := True;
end;

procedure TMainForm.EditUsernameChange(Sender: TObject);
begin
  (* Wenn Benutzername gendert wird, muss RasDialParams aktualisiert werden *)
  StrPCopy(RasDialParams.szUserName, EditUsername.Text);
end;

function TMainForm.Status(var entryname: String) : Boolean;
var rasconnArray: TRasConn;
    lpcb, lpcConnections: Longint;
begin
  (* rasconnArray vorbereiten *)
  FillChar(rasconnArray, SizeOf(TRasConn), 0);
  rasconnArray.dwSize := SizeOf(TRasConn);
  lpcb := SizeOf(TRasConn);
  lpcConnections := 0;

  (* Aktive Verbindungen holen *)
  error := RasEnumConnections(@rasconnArray, lpcb, lpcConnections);

  (* War abfrage erfolgreich? *)
  if (error <> SUCCESS) then begin
    //ShowMessage('Fehler bei der Statusabfrage');
    Result := False;
    Exit;
  end;

  (* Bestehen nun Verbindungen? *)
  if (lpcConnections > 0) then begin
    (* jaaa *)
    entryname := rasconnArray.szEntryName;
    hRAS := rasconnArray.hrasconn;
    Result := True;
  end else begin
    (* nein *)
    entryname := '';
    hRAS := 0;
    Result := False;
  end;

end;

function TMainForm.Hangup : Boolean;
var stat: Boolean;
    name: String;
begin
  (* Besteht berhaupt eine Verbindung?
   * wenn ja, wird hRAS neu besetzt
   *)
  stat := Status(name);
  if (stat = True) then begin
    error := RasHangUp(hRAS);
    if (error <> SUCCESS) then begin
      //ShowMessage('Fehler beim Verbindungsabbau der Verbindung: ' + name);
      Result := False;
      Exit;
    end;
    Result := True;
    Exit;
  end;
  error := ERROR_NO_CONNECTION;
  Result := False;
end;

procedure TMainForm.Beenden1Click(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.HTTPServerClientConnect(Sender: TObject;
  Socket: TCustomWinSocket);
var s: string;
begin
  (* Hier muss die IP-Adresse berprft werden...
   * erlaubt sind folgende:
   * - 192.168.x.x
   * - 127.x.x.x
   *)
  s := Socket.RemoteAddress;

  if (Copy(s, 0, 8) = '192.168.') then begin
    (* erlaube Zugriff fr 192.168.xxxx*)
  end else if (Copy(s, 0, 4) = '127.') then begin
    (* erlaube Zugriff fr 127.xxxx *)
  end else begin
    (* Verbiete Zugriff! fr alle anderen Adressen *)
    Socket.Close;
  end;

  (* Zeilenspeicher initialisieren *)
  indata := '';

end;

procedure TMainForm.BtnServerStartClick(Sender: TObject);
begin
  (* Server konfigurieren *)
  HTTPServer.Port := StrToInt(EditPort.Text);
  (* Server starten=aktivieren *)
  HTTPServer.Open;
  (* Statusmeldung *)
  ServerStatusLabel.Caption := 'Server luft'#13#10'Port: '+EditPort.Text;
end;

procedure TMainForm.BtnServerStopClick(Sender: TObject);
begin
  (* Server stoppen *)
  HTTPServer.Close;
  (* Statusmeldung *)
  ServerStatusLabel.Caption := 'Server luft nicht';
end;

procedure TMainForm.HTTPServerClientRead(Sender: TObject;
  Socket: TCustomWinSocket);
var s: String;
    sl: TStringList;
begin
  (* Daten kommen vom Client... *)
  (* Empfange Daten *)
  s := Socket.ReceiveText;
  (* Dem Zeilenspeicher hinzufgen *)
  indata := indata + s;
  (* Prfen, ob indata mit #10=newline endet
   * Dann wurde ein Befehl komplett empfangen...
   * HINWEIS: DER SERVER HLT SICH NICHT STRENG AN HTTP!!!!
   *)
  if (indata[Length(indata)] = #10) then begin
    (* die erste Zeile von indata ist wichtig,
     * hier steht das GET-Kommando
     *)
    sl := TStringList.Create;
    sl.Text := indata;
    (* indata erhlt jetzt nur die erste Zeile *)
    indata := sl.Strings[0];
    (* "GET " entfernen *)
    indata := Copy(indata, 5, Length(indata));
    (* den Schluss von indata entfernen ( HTTP/1.1 ) *)
    indata := Copy(indata, 0, Pos(' ', indata)-1);
    (* in indata steht jetzt die Website, die aufgerufen wurde *)
    if (indata = '/') or (indata = '/status') then begin
      (* Status-Seite anzeigen ... *)
      SendStatus(Socket);
    end else if (indata = '/online') then begin
      (* Einwhlen + neuer Status *)
      SendOnline(Socket);
    end else if (indata = '/offline') then begin
      (* Auflegen  + neuer Status *)
      SendOffline(Socket);
    end else if (indata = '/about') then begin
      (* Copyright by adabolo... *)
      SendAbout(Socket);
    end else begin
      (* File not found *)
      SendFileNotFound(Socket);
    end;

    (* Verbindung beenden *)
    Socket.Close;
    (* Zeilenspeicher initialisieren (sicher ist sicher,
     * wird zwar in OnClientConnect auch gemacht, aber ...
     *)
    indata := '';
  end;
end;

procedure TMainForm.SendStatus(Socket: TCustomWinSocket);
var connected: Boolean;
    entryname: String;
begin
  (* DF-Status abfragen *)
  connected := Status(entryname);


  (* Ich verzichte hier auf eine HTTP-Antwort und schicke
   * gleich die html-Daten
   * dito SendOnline, SendOffline, SendAbout, SendFileNotFount
   *)
  with Socket do begin
    SendText('<html>'#13#10);
    SendText('<head>'#13#10);
    SendText('  <title>RAS Manager - Status</title>'#13#10);
    SendText('</head>'#13#10);
    SendText('<body>'#13#10);
    SendText('<h1>RAS Manager - Status</h1>'#13#10);
  end;

  if (connected = True) then begin
    (* Verbunden ... *)
    Socket.SendText('<h2>Server ist <font color="red"><b>online</b></font></h2>'#13#10);
    Socket.SendText('Verbunden mit "' + entryname + '"'#13#10);
    Socket.SendText('<h2><a href="/offline">Offline gehen</a></h2>'#13#10);
  end else begin
    (* Nicht verbunden ... *)
    Socket.SendText('<h2>Server ist <font color="green"><u>offline</u></font></h2>'#13#10);
    Socket.SendText('<h2><a href="/online">Online gehen</a></h2>'#13#10);
    Socket.SendText('Es wird die Verbindung "' + RasDialParams.szEntryName
                    + '" verwendet'#13#10);
  end;

  with Socket do begin
    SendText('</body>'#13#10);
    SendText('</html>'#13#10);
  end;
end;

procedure TMainForm.SendOnline(Socket: TCustomWinSocket);
var ret, connected: Boolean;
    dial_error: Integer;
    entryname: String;
begin
  (* erstmal whlen versuchen *)
  ret := Dial;
  (* und eventueller Fehlercode sichern *)
  dial_error := error;

  (* Ausgaben starten *)
  with Socket do begin
    SendText('<html>'#13#10);
    SendText('<head>'#13#10);
    SendText('  <title>RAS Manager - Online gehen</title>'#13#10);
    SendText('</head>'#13#10);
    SendText('<body>'#13#10);
    SendText('<h1>RAS Manager - Online gehen</h1>'#13#10);
  end;


  (* Wahl erfolgreich?? *)
  if (ret = True) then begin
    (* jaa *)
     Socket.SendText('Die Einwahl war erfolgreich'#13#10);
  end else begin
    (* nicht erfolgreich *)
     Socket.SendText('Die Einwahl war <i>nicht</i> erfolgreich!<br>'#13#10);
     Socket.SendText('Fehler: ' + IntToStr(dial_error) + ': '
                        + getRASError(dial_error));
  end;


  (* eine Sekunde warten, damit auch der aktuelle Status abgerufen wird *)
  Sleep(1000);
  (* besteht jetzt eine Verbindung? *)
  connected := Status(entryname);
  (* neuen Status ausgeben *)
  if (connected = True) then begin
    (* Verbunden ... *)
    Socket.SendText('<h2>Server ist <font color="red"><b>online</b></font></h2>'#13#10);
    Socket.SendText('Verbunden mit "' + entryname + '"'#13#10);
    Socket.SendText('<h2><a href="/offline">Offline gehen</a></h2>'#13#10);
  end else begin
    (* Nicht verbunden ... *)
    Socket.SendText('<h2>Server ist <font color="green"><u>offline</u></font></h2>'#13#10);
    Socket.SendText('<h2><a href="/online">Online gehen</a></h2>'#13#10);
    Socket.SendText('Es wird die Verbindung "' + RasDialParams.szEntryName
                    + '" verwendet'#13#10);
  end;

  (* zurck *)
  Socket.SendText('<p><a href="/">Zurck</a> zur Status-Seite'#13#10);

  (* Ende... *)
  with Socket do begin
    SendText('</body>'#13#10);
    SendText('</html>'#13#10);
  end;

end;

function TMainForm.getRASError(code : Integer) : String;
var errorString : Array[0..100] of Char;
begin
  error := RasGetErrorString(code, errorString, 100);
  Result := errorString;
end;

procedure TMainForm.SendOffline(Socket: TCustomWinSocket);
var ret, connected: Boolean;
    entryname: String;
begin
  (* erst Status-Abfrage: Sind wir berhaupt online? *)
  connected := Status(entryname);

  (* Ausgaben starten *)
  with Socket do begin
    SendText('<html>'#13#10);
    SendText('<head>'#13#10);
    SendText('  <title>RAS Manager - Offline gehen</title>'#13#10);
    SendText('</head>'#13#10);
    SendText('<body>'#13#10);
    SendText('<h1>RAS Manager - Offline gehen</h1>'#13#10);
  end;

  (* also: sind wir online? *)
  if (connected = True) then begin
    (* Versuche, die Verbindung zu beenden *)
    ret := Hangup;
    (* erfolgreich? *)
    if (ret = True) then begin
      Socket.SendText('Verbindung erfolgreich abgebaut.'#13#10);
    end else begin
      (* Fehlercode ausgeben *)
      Socket.SendText('Es ist ein Fehler aufgetreten!<br>'#13#10);
      Socket.SendText('Fehler: ' + IntToStr(error) + ': '
                        + getRASError(error));
    end;
  end else begin
    (* es besteht gar keine Verbindung *)
    Socket.SendText('Es ist ein Fehler aufgetreten!<br>'#13#10);
    Socket.SendText('Fehler: Es besteht gar keine Verbindung.'#13#10);
  end;


  (* eine Sekunde warten, damit auch der aktuelle Status abgerufen wird *)
  Sleep(1000);
  (* neuen Status ausgeben *)
  connected := Status(entryname);
  if (connected = True) then begin
    (* Verbunden ... *)
    Socket.SendText('<h2>Server ist <font color="red"><b>online</b></font></h2>'#13#10);
    Socket.SendText('Verbunden mit "' + entryname + '"'#13#10);
    Socket.SendText('<h2><a href="/offline">Offline gehen</a></h2>'#13#10);
  end else begin
    (* Nicht verbunden ... *)
    Socket.SendText('<h2>Server ist <font color="green"><u>offline</u></font></h2>'#13#10);
    Socket.SendText('<h2><a href="/online">Online gehen</a></h2>'#13#10);
    Socket.SendText('Es wird die Verbindung "' + RasDialParams.szEntryName
                    + '" verwendet'#13#10);
  end;

  (* Zurck... *)
  Socket.SendText('<p><a href="/">Zurck</a> zur Status-Seite'#13#10);

  (* Ende *)
  with Socket do begin
    SendText('</body>'#13#10);
    SendText('</html>'#13#10);
  end;
end;

procedure TMainForm.SendAbout(Socket: TCustomWinSocket);
begin
  with Socket do begin
    SendText('<html>'#13#10);
    SendText('<head>'#13#10);
    SendText('  <title>RAS Manager - Info</title>'#13#10);
    SendText('</head>'#13#10);
    SendText('<body>'#13#10);
    SendText('<h1>RAS Manager - Info</h1>'#13#10);
    SendText('Version 1.0 (2003/03/19)<br>'#13#10);
    SendText('&copy 2003 by Andreas Dangel '
          + '&lt;<a href="mailto:adabolo@adabolo.de">adabolo@adabolo.de</a>'
          + '&gt;<br>'#13#10);
    SendText('under the GNU GPL v2'#13#10);
    SendText('<p>more infos - visit: <a href="http://www.adabolo.de">adabolo.de</a>'#13#10);
    SendText('<h2><a href="/">Zur Status-Seite</a></h2>'#13#10);
    SendText('</body>'#13#10);
    SendText('</html>'#13#10);
  end;
end;

procedure TMainForm.SendFileNotFound(Socket: TCustomWinSocket);
begin
  with Socket do begin
    SendText('<html>'#13#10);
    SendText('<head>'#13#10);
    SendText('  <title>RAS Manager - File Not Found</title>'#13#10);
    SendText('</head>'#13#10);
    SendText('<body>'#13#10);
    SendText('<h1>RAS Manager - File Not Found</h1>'#13#10);
    SendText('Die angegebene Adresse wurde nicht gefunden!'#13#10);
    SendText('<h2><a href="/">Zurck zur Status-Seite</a></h2>'#13#10);
    SendText('<a href="/about">about</a>'#13#10);
    SendText('</body>'#13#10);
    SendText('</html>'#13#10);
  end;
end;

procedure TMainForm.BtnCancelClick(Sender: TObject);
begin
  (* keine Einstellungen speichern *)
  (* sondern die alten laden *)
  LoadSettings;

  (* und Fenster verstecken *)
  Hide;
end;

procedure TMainForm.BtnOKClick(Sender: TObject);
begin
  (* Einstellungen speichern *)
  SaveSettings;

  (* Server neu starten (wegen eventueller Port-nderung) *)
  BtnServerStopClick(Sender);
  BtnServerStartClick(Sender);

  (* Fenster verstecken *)
  Hide;
end;

procedure TMainForm.SaveSettings;
var ini : TIniFile;
    index,i : Integer;
    pw, pworig: String;
begin
  index := LastDelimiter('\', Application.ExeName);
  ini := TIniFile.Create(Copy(Application.ExeName, 0, index) + 'settings.ini');

  (* speichern: *)
  (* chkBox Autostart... *)
  ini.WriteBool('main', 'autostart', chkAutostart.Checked);
  (* Port *)
  ini.WriteString('main', 'port', EditPort.Text);
  (* DF-Verbindung *)
  ini.WriteString('main', 'entryname', BoxConnections.Items[BoxConnections.ItemIndex]);
  (* Username *)
  ini.WriteString('main', 'username', EditUsername.Text);
  (* Passwort
      sollte "verschlsselt" werden
      wenigstens pseudo-verschlsselung
   *)
  pw := ' ';
  pw[1] := Char(32 + Random(32));
  pworig := EditPassword.Text;
  for i := 1 to Length(pworig) do begin
    pw := pw + String(Char(Integer(pworig[i]) + Integer(pw[1])));
  end;
  ini.WriteString('main', 'password', pw);

  ini.Free;
end;

procedure TMainForm.LoadSettings;
var ini: TIniFile;
    index, i: Integer;
    entryname, pw, pworig, user: String;
begin
  index := LastDelimiter('\', Application.ExeName);
  ini := TIniFile.Create(Copy(Application.ExeName, 0, index) + 'settings.ini');

  (* laden: *)
  (* chkBox Autostart *)
  chkAutostart.Checked := ini.ReadBool('main', 'autostart', False);
  (* Port *)
  EditPort.Text := ini.ReadString('main', 'port', '80');
  (* DF-Verbindung *)
  entryname := ini.ReadString('main', 'entryname', '');
  for i := 0 to BoxConnections.Items.Count-1 do begin
    if (entryname = BoxConnections.Items[i]) then begin
      BoxConnections.ItemIndex := i;
      BoxConnectionsChange(nil);
      Break;
    end;
  end;
  (* Username *)
  user := ini.ReadString('main', 'username', '');
  if (user <> '') then begin
    EditUsername.Text := user;
    EditUsernameChange(nil);
  end;
  (* Password
     muss wieder "entschlsselt" werden...
  *)
  pw := ini.ReadString('main', 'password', '0');
  pworig := '';
  for i := 2 to Length(pw) do begin
    pworig := pworig + String(Char(Integer(pw[i]) - Integer(pw[1])));
  end;
  EditPassword.Text := pworig;

  (* fertig *)
  ini.Free;
end;

end.
